<?php

use Monolog\Logger;

class FileUtils
{
  private $log;
  private $config;
  private $applicationDirectory;

  public function __construct(Logger $log, $config)
  {
    $this->log = $log;
    $this->config = $config;
    $this->init();
  }

  private function init()
  {
    // By default production
      $this->applicationDirectory = "/var/www/vhosts/hausverwaltung-kappel.de/httpdocs/app";
    // For localhost on Windows with XAMPP.
    if ($this->config['env'] == 'local') {
      $this->applicationDirectory = "C:\\xampp\htdocs\\steinle-hv-admin\\app";
    } else if ($this->config['env'] == 'dev') {
      $this->applicationDirectory = "/usr/www/users/" . get_current_user() . "/kappel";
    }
  }

  public function loadFilesFromFolder($folder, $extension = 'xml'): ?array
  {
    $ext = "*." . $extension;
    return glob($folder . $ext);
  }

  public function openFile($file, $extension = 'xml')
  {
    if (is_file($file) && pathinfo($file, PATHINFO_EXTENSION) == $extension) {
      $fp = @fopen($file, "rb");
      $data = @fread($fp, filesize($file));
      @fclose($fp);
      return $data;
    }
    return false;
  }

  public function deleteFilesInFolder($folder, $daysToKeep = 30)
  {
    if (file_exists($folder)) {
      foreach (new DirectoryIterator($folder) as $fileInfo) {
        if ($fileInfo->isDot()) {
          continue;
        }
        if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= $daysToKeep * 24 * 60 * 60) { //30days
          unlink($fileInfo->getRealPath());
          $this->log->info("Datei gelöscht, da älter als " . $daysToKeep . " Tage: " . $fileInfo->getRealPath() . "<br>");
        }
      }
    }
  }

  /**
   *  create folder if it does not exist
   *  => create empty text file
   *  => create empty file with the given fileName
   *
   * @param $houseId
   * @param $fileName
   * @return bool
   */
  public function createStaticHouseInformation($houseId, $fileName): bool
  {
    $dir = $this->applicationDirectory . "/static/houses/" . $houseId;

    // If folder does not exist -> create
    if (!file_exists($dir)) {

      $file_info = 'infos.txt';
      $content_info = "Hausverwaltung Kappel;09072-91900";
      $file_name = $fileName;

      if (is_dir($dir) === false) {
        mkdir($dir);
      }

      $file = fopen($dir . '/' . $file_info, "w");
      fwrite($file, $content_info);
      fclose($file);

      touch($dir . '/' . utf8_decode($file_name));

      return true;

    }

    return false;
  }

  public function setStaticHouseIdentifier($houseId, $fileName): bool
  {
    $dir = $this->applicationDirectory . "/static/houses/" . $houseId;

    $files = scandir($dir);
    if (!$files) {
      return false;
    }

    // Remove all previous files. Do not touch txt files and '.' '..' links
    foreach ($files as $file) {
      $ext = pathinfo($file, PATHINFO_EXTENSION);
      if ($ext != 'txt' && $file != '.' && $file != '..') {
        if (file_exists($dir . '/' . $file)) {
          unlink($dir . '/' . $file);
        }
      }
    }

    // Remove illegal chars for filenames
    $illegalCharacters = array("/", ":", "|", "\\", "?", "*", "&", "<", "\"", ";", ">", "[", "]", "=");
    $fileName = str_replace($illegalCharacters, "_", $fileName);

    // Replace 'Umlaute'
    $replacePairs = array(
      'ä' => 'ae',
      'ö' => 'oe',
      'ü' => 'ue',
      'é' => 'e'
    );
    $fileName = strtr($fileName, $replacePairs);

    $filePath = $dir . '/' . $fileName . '.info';
    $this->log->info("Erstelle Datei: " . $filePath, ['detail' => true]);
    if (!touch($filePath)) {
      $this->log->warning("Fehler beim Erstellen der Datei: " . $filePath . " (Haus=" . $houseId . ")");
    }

    return true;
  }
}
