<?php

/**
 * Offen:
 * - Datenbank sichern
 * - Mail schicken
 * - Logs weiter überprüfen (Nicht alle Metadaten werden geloggt)
 * - Was passiert alles bei force_update?
 */

require_once '../parser/vendor/autoload.php';

require_once '../parser/XmlParserLogHandler.php';
require_once '../parser/FileUtils.php';
require_once '../parser/OpenImmoXmlParser.php';
require_once '../parser/Comparator.php';
require_once '../parser/UpdateTable.php';
require_once '../parser/legacy/ClientClass.php';
require_once '../parser/legacy/ClientUpdate.php';
require_once '../parser/legacy/LegacyClientConverter.php';
require_once '../../config/database.php';
require_once '../../models/client.php';
require_once '../../models/backup.php';
require_once '../helper/DatabaseBackup.php';

$config = include('../../config/config.php');

$log = new Monolog\Logger('parser');
$logHandler = ($config['log_level'] == 'debug') ? new XmlParserLogHandler(Monolog\Logger::DEBUG) : new XmlParserLogHandler(Monolog\Logger::INFO);
$log->pushHandler($logHandler);

$fileUtils = new FileUtils($log, $config);
$xmlFiles = $fileUtils->loadFilesFromFolder('../xml/');

if (!array_key_exists(0, $xmlFiles)) {
  $log->info("Nichts zum Parsen gefunden!");
  return;
}

$parser = new OpenImmoXmlParser($log);
$converter = new LegacyClientConverter($log);

$db = new Database();
$dbConnection = $db->connect();
$dbApi = new Client($dbConnection);

// Backup database
if (sizeof($xmlFiles) > 0) {
  $dbBackup = new DatabaseBackup($log);
  $dbBackup->backup();
}

/* Forced update does the following:
  - Processes all *.xml files in xml folder
  - Creates/updates the DB house (WEG)
  - Force delete/create the static house identifier file in static/house/[OBJECTTITLE]
  - Iterate over all objects and create/update the objects
      Iterate over all flats and clients and update correspondence table for 'objectid'
      (pid, hid, flat, level, isCouncil are checked for equality)
  - Create + show the update table if changes are > 0
*/
if ($config['parser']['force_update'] == true) {

  foreach ($xmlFiles as $xmlFile) {

    $log->info("Lese folgende Datei ein: <b>" . $xmlFile . "<b>");

    $xmlData = $fileUtils->openFile($xmlFile);
    $realEstateModel = $parser->parse($xmlData);

    $log->info("Meta Daten:", ['detail' => true,
      'params' => [
        'Sendersoftware ' => $realEstateModel->getRootInfos()->getSenderSoftware(),
        'Version ' => $realEstateModel->getRootInfos()->getVersion()
      ]]);

    $legacyClients = $converter->convertRealEstateToClientArray($realEstateModel);
    $log->info("Insgesamt " . sizeof($legacyClients) . " Clients bei " . sizeof($realEstateModel->getHouses()) .
      "Häusern", ['detail' => true]);

    // Create // update the WEG
    $dbApi->create_house_xml($realEstateModel->getObjectId(), $realEstateModel->getTitle(), $realEstateModel->getZip(), $realEstateModel->getCity());

    // Recreate the static info house identifier for the WEG
    $fileUtils->setStaticHouseIdentifier($realEstateModel->getObjectId(), $realEstateModel->getTitle());

    // Iterate over the real estates houses
    foreach ($realEstateModel->getHouses() as $house) {
      // Create / update objects
      $dbApi->create_object($realEstateModel->getObjectId(), $house);

      // Update the objectId
      foreach ($house->getFlats() as $flat) {
        foreach ($flat->getClients() as $client) {
          // e.g. 32402
          $dbApi->house_id = $realEstateModel->getObjectId();
          // houseId <-> objectId
          $dbApi->object_id = $house->getHouseId();
          $dbApi->pid = $client->getId();
          $dbApi->flat = $flat->getNumber();
          $dbApi->level = $client->getLevel();
          $dbApi->isCouncil = $client->getIsCouncil();

          if (!$dbApi->update_objectid()) {
            $log->error("Error setting objectId for " . $client->__toString() . " <br> " . $flat->__toString());
          }
        }
      }
    }

    // Log the Details
    echo($logHandler->getFormattedDetailLog());

    // Compare XML with database
    $comparator = new Comparator($dbApi);
    $updateData = $comparator->compareParsedClientsWithDatabase($legacyClients, $realEstateModel->getObjectId());

    // Create and draw "client" update table
    $updateTable = new UpdateTable($log, $realEstateModel->getObjectId(), $realEstateModel->getTitle());
    // Only print update table if >0 changes
    if (! empty($updateData)) {
      $updateTable->drawClientUpdateTable($updateData);
    }

    $log->info("<br>-----------------------------------------------------------------------------------  <br>");
  }

  // In manual force mode exit here.
  exit();
}


// Only process the first file
$xmlFile = $xmlFiles[0];
$xmlData = $fileUtils->openFile($xmlFile);
$realEstateModel = $parser->parse($xmlData);
$legacyClients = $converter->convertRealEstateToClientArray($realEstateModel);

$log->info("Lese folgende Datei ein: <b>" . $xmlFile . "<b>");

$log->info("<b><br>Erstelle/Aktualisiere Wohnanlage/Objekt...</b>");

$log->info("Meta Daten:", ['detail' => true,
  'params' => [
    'Sendersoftware ' => $realEstateModel->getRootInfos()->getSenderSoftware(),
    'Version ' => $realEstateModel->getRootInfos()->getVersion()
  ]]);

$log->info("Insgesamt " . sizeof($legacyClients) . " Clients bei " . sizeof($realEstateModel->getHouses()) .
  "Häusern", ['detail' => true]);


/***** Create / Update House *****/

// Update: set title as name (more descriptive), street and number empty because we don't know if the WEG has several
// streets and numbers
if ($dbApi->create_house_xml($realEstateModel->getObjectId(), $realEstateModel->getTitle(), $realEstateModel->getZip(), $realEstateModel->getCity())) {

  $log->info("<br><font color=green><b>OK:</font></b> Wohnanlage/Objekt \"" . $realEstateModel->getTitle() .
    " " . $realEstateModel->getZip() . " " . $realEstateModel->getCity() . "\" erstellt!", ['detail' => true]);


  $fileUtils->createStaticHouseInformation($realEstateModel->getObjectId(), $realEstateModel->getTitle());

} else {

  $log->error("<br><font color=red><b>FEHLER:</font></b> Wohnanlage/Objekt \"" . $realEstateModel->getTitle() .
    " " . $realEstateModel->getZip() . " " . $realEstateModel->getCity() . "\" nicht erstellt!<br>", ['detail' => true]);
}

/***** Create / Update Objects  *****/

foreach ($realEstateModel->getHouses() as $house) {
  $dbApi->create_object($realEstateModel->getObjectId(), $house);
}

/***** Log all details from parsing *****/

echo($logHandler->getFormattedDetailLog());

/***** Print overall result. If > 0 error occurred log here. *****/

if ($logHandler->getErrorCount() == 0) {
  $log->info("<b>RESULTAT:</b> <font color=green><b>ERFOLGREICH!</b></font>");
} else {
  $log->info("<b>RESULTAT: </b><font color=red><b>FEHLER</b></font>! Dem Admin melden: Es lagen ". $logHandler->getErrorCount() ." Fehler vor.");
}
$log->info("<br>");

/***** Compare XML with DB and populate a table with potential needed updates *****/

$comparator = new Comparator($dbApi);
$updateData = $comparator->compareParsedClientsWithDatabase($legacyClients, $realEstateModel->getObjectId());

$updateTable = new UpdateTable($log, $realEstateModel->getObjectId(), $realEstateModel->getTitle());
// Only print update table if >0 changes
if (! empty($updateData)) {
  $updateTable->drawClientUpdateTable($updateData);
}

// Do some cleanup
$fileUtils->deleteFilesInFolder("../xml");
$fileUtils->deleteFilesInFolder("../mail");
$fileUtils->deleteFilesInFolder("../backup_db", 90);

// Rename the XML file
if ($logHandler->getErrorCount() == 0) {
  $log->info("Verarbeitete Datei gesichert unter: <b>" . $xmlFile . ".bak <b>");
  rename($xmlFile, $xmlFile . ".bak");
}

$log->info("<br>Neue Datei-Suche wird eingleitet...<br>");

