<?php 
/**
 * This class is used as an interface to the database.
 * It contains the entire API to communicate with the OBJECT table.
 * This should be used for the REST API between server and clients.
 *
 * @copyright  2021 FD
 * @version    Release: 2.00
 */ 

  class Obj {

    // {{{ variables
    
    // DB stuff
    private $conn;
    private $tableObject = 'object';
    private $tableHouse = 'house';
    private $tableClient = 'client';
    private $tableCorres = 'correspondence';

    // Object Properties
    public $oid; 
    public $hid; 
    public $title;
    public $street;
    public $number;
    public $zip;
    public $city;
    public $created_at;

    // }}}
    // {{{ functions

    /**
     * Constructor with DB
     *
     * @param PDO $db the database connection object
     */
    public function __construct( $db ) {
      $this->conn = $db;
    }


    /**
     * GET object by hid
     *
     * Required: @this->hid
     *
     * @return array of rows of oid, hid, title, street, number, zip, city, created_at 
     *         NULL if there is an error
     */
    public function get_object_by_hid() {
      // Create query
      $query = 'SELECT oid, hid, title, street, number, zip, city, created_at
                FROM ' . $this->tableObject . '
                WHERE hid = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hid = htmlspecialchars(strip_tags($this->hid));

      // Bind ID
      $stmt->bindParam( 1, $this->hid );

      // Execute query
      if( $stmt->execute() ) {

        $objects = null;

        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            $objects[] = $row;
        }

        return $objects;
      }

      // Print error if something goes wrong
      error_log( "ERROR: object.get_object_by_hid: " . $stmt->error , 0);

      return null;
    }


    /**
     * CREATE object item
     *
     * @return TRUE if insertion was successful
     *         FALSE if there is an error
     */
    public function create_object() {
      // Create query
      $query = 'INSERT INTO ' . $this->tableObject . '
                SET hid = :hid, title = :title, street = :street, number = :number, zip = :zip, city = :city';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hid = htmlspecialchars(strip_tags($this->hid));
      $this->title = htmlspecialchars(strip_tags($this->title));
      $this->street = htmlspecialchars(strip_tags($this->street));
      $this->number = htmlspecialchars(strip_tags($this->number));
      $this->zip = htmlspecialchars(strip_tags($this->zip));
      $this->city = htmlspecialchars(strip_tags($this->city));

      // Bind ID
      $stmt->bindParam( ':hid', $this->hid );
      $stmt->bindParam( ':title', $this->title );
      $stmt->bindParam( ':street', $this->street );
      $stmt->bindParam( ':number', $this->number );
      $stmt->bindParam( ':zip', $this->zip );
      $stmt->bindParam( ':city', $this->city );

      // Execute query
      if( $stmt->execute() ) {

          return true;
      }

      // Print error if something goes wrong
      error_log( "ERROR: object.create_object: " . $stmt->error , 0);

      return false;
    }


    /**
     * DELETE object item by oid
     *
     * Required: @this->hid
     *
     * @return TRUE if deletion was successful
     *         FALSE if there is an error
     */
    public function delete_object_by_oid() {
      // Create query
      $query = 'DELETE FROM ' . $this->tableObject . ' 
                WHERE oid = :oid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->oid = htmlspecialchars(strip_tags($this->oid));

      // Bind data
      $stmt->bindParam(':oid', $this->oid );

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      error_log( "ERROR: object.delete_object_by_oid: " . $stmt->error , 0);

      return false;
    }


    /**
     * UPDATE object by oid
     *
     * Required: @this->oid
     *
     * @return TRUE if update was successful
     *         FALSE if there is an error
     */
    public function update_object_by_oid() {
      // Create query
      $query = 'UPDATE ' . $this->tableObject . '
                SET hid = :hid, title = :title, street = :street, number = :number, zip = :zip, city = :city
                WHERE oid = :oid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hid = htmlspecialchars(strip_tags($this->hid));
      $this->title = htmlspecialchars(strip_tags($this->title));
      $this->street = htmlspecialchars(strip_tags($this->street));
      $this->number = htmlspecialchars(strip_tags($this->number));
      $this->zip = htmlspecialchars(strip_tags($this->zip));
      $this->city = htmlspecialchars(strip_tags($this->city));

      // Bind data
      $stmt->bindParam(':hid', $this->hid);
      $stmt->bindParam(':title', $this->title);
      $stmt->bindParam(':street', $this->street);
      $stmt->bindParam(':number', $this->number);
      $stmt->bindParam(':zip', $this->zip);
      $stmt->bindParam(':city', $this->city);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      error_log( "ERROR: object.update_object_by_oid: " . $stmt->error , 0);

      return false;
    }

    // }}}
}